defmodule ParserTest do
  use ExUnit.Case
  doctest Indian.Parser

  test "simple" do
    assert {:ok, ["GET", "/foobar", "1.1", _, _], _, _, _, _} =
      Indian.Parser.request("GET /foobar HTTP/1.1\r\n\r\n") 
  end

  test "headers" do
    assert {:ok, ["GET", "/foobar", "1.1", [["User-Agent", "tester"], ["Foo", "bar"]], _], _, _, _, _} =
      Indian.Parser.request("GET /foobar HTTP/1.1\r\nUser-Agent:tester\r\nFoo:    bar\r\n\r\n") 
  end

  test "body_without_headers" do
    assert {:ok, ["GET", "/foobar", "1.1", [], "hello"], _, _, _, _} =
      Indian.Parser.request("GET /foobar HTTP/1.1\r\n\r\nhello") 
  end

  test "body_with_headers" do
    assert {:ok, ["GET", "/foobar", "1.1", [["User-Agent", "tester"], ["Foo", "bar"]], "hello"], _, _, _, _} =
      Indian.Parser.request("GET /foobar HTTP/1.1\r\nUser-Agent:tester\r\nFoo:    bar\r\n\r\nhello") 
  end

  test "empty_body_with_headers" do
    assert {:ok, ["GET", "/foobar", "1.1", [["User-Agent", "tester"], ["Foo", "bar"]], ""], _, _, _, _} =
      Indian.Parser.request("GET /foobar HTTP/1.1\r\nUser-Agent:tester\r\nFoo:    bar\r\n\r\n") 
  end

  test "empty_body_without_headers" do
    assert {:ok, ["GET", "/foobar", "1.1", [], ""], _, _, _, _} =
      Indian.Parser.request("GET /foobar HTTP/1.1\r\n\r\n") 
  end

  test "headers_with_spaces" do
    assert {:ok, _, _, _, _, _} =
      Indian.Parser.request("GET /date HTTP/1.1\r\nUser-Agent: check_http/v2.3.1 (monitoring-plugins 2.3.1)\r\n\r\n")
  end
  
  test "wrong_headers" do
    assert {:error, _, _, _, _, _} =
      Indian.Parser.request("GET /foobar HTTP/1.1\r\nUser-Agenttester\r\nFoo:    bar\r\n\r\n") 
  end

  test "no_version" do
    assert {:error, _, _, _, _, _} =
      Indian.Parser.request("GET /foobar\r\n\r\n") 
  end
    
  test "no_path" do
    assert {:error, _, _, _, _, _} =
      Indian.Parser.request("GET\r\n\r\n") 
  end
    
  test "no_empty_line" do
    assert {:error, _, _, _, _, _} =
      Indian.Parser.request("GET /foobar HTTP/1.1\r\n") 
  end

  test "unknown_method" do # Will be dealt with at a higher level
    assert {:ok, ["FOOBAR", "/", "1.0", _, _], _, _, _, _} =
      Indian.Parser.request("FOOBAR / HTTP/1.0\r\n\r\n") 
  end
    
end
